/*
 *
 * X DLX Simulator
 *
 * The DLX is a 32-bit RISC machine based on the MIPS 2000 architecture.
 * The architecture includes 32 32-bit general purpose registers and a 
 * program counter.
 *
 * Floating point registers and an interrupt register are not implemented.
 *
 * The address space is 4 Gbytes, big-endian byte-addressable using 32-bit
 * addresses. All instructions must be properly aligned.
 *
 *
 * Cameron Humphries
 * Department of Computer Science
 * University of Adelaide
 * 1992
 *
 */
 
#include <stdio.h>
#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>

#include "sim.h"
#include "/usr/local/X11.contrib/include/Xfwf/FileComp.h"

#define NO_MODE 0
#define OPEN_MODE 1
#define SAVE_MODE 2

static void new_data();
static void open_data();
static void save_data();
static void saveas_data();
static void quit_data();

static void open_ok();
static void open_cancel();

extern int sim_pid;
extern int num_kb;
extern char *dlx_filename;
extern Widget code_list,data_list;

int mode = NO_MODE;

Widget filename_popup;
/*
 * Create menu button for popup file menu.
 */
void Create_File_Menu(Widget form)
{
  Widget w,file_menu_popup;
  static Arg arg_list[] = {
    {XtNlabel, (XtArgVal)"FILE"},
    {XtNmenuName, (XtArgVal)"File_Menu_Popup"},
  };
  static Arg menu_entry_args[] = {
    {XtNlabel, (XtArgVal)NULL},
    {XtNcallback, (XtArgVal)NULL},
  };
  static Arg open_args[] = {
    {XtNcallback,(XtArgVal)NULL},
    {XtNcancelCallback,(XtArgVal)NULL},
  };

  static XtCallbackRec new_cbks[] = {
    {new_data, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec open_cbks[] = {
    {open_data, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec save_cbks[] = {
    {save_data, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec saveas_cbks[] = {
    {saveas_data, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec quit_cbks[] = {
    {quit_data, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec open_ok_cbks[] = {
    {open_ok, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec open_cancel_cbks[] = {
    {open_cancel, NULL},
    {NULL,NULL}
  };
  
  /* Create Popup File Open Widget */
  filename_popup = XtCreatePopupShell("File_Name_Popup",overrideShellWidgetClass,
				       form, NULL, 0);

  open_args[0].value = (XtArgVal)open_ok_cbks;
  open_args[1].value = (XtArgVal)open_cancel_cbks;
  w = XtCreateManagedWidget("Open_Select",xfwffileCompleteWidgetClass,filename_popup,
			    open_args,XtNumber(open_args));
			    
  /* Create Popup Menu */
  file_menu_popup = XtCreatePopupShell("File_Menu_Popup",simpleMenuWidgetClass,
				       form, NULL, 0);

  /* Create Menu entries */
  menu_entry_args[0].value = (XtArgVal)"New";
  menu_entry_args[1].value = (XtArgVal)new_cbks;
  w = XtCreateManagedWidget("FM_New",smeBSBObjectClass,file_menu_popup,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  menu_entry_args[0].value = (XtArgVal)"Open";
  menu_entry_args[1].value = (XtArgVal)open_cbks;
  w = XtCreateManagedWidget("FM_Open",smeBSBObjectClass,file_menu_popup,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  w = XtCreateManagedWidget("Line_1",smeLineObjectClass,file_menu_popup,
			    NULL,0);

  menu_entry_args[0].value = (XtArgVal)"Save";
  menu_entry_args[1].value = (XtArgVal)save_cbks;
  w = XtCreateManagedWidget("FM_Save",smeBSBObjectClass,file_menu_popup,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  menu_entry_args[0].value = (XtArgVal)"Save As";
  menu_entry_args[1].value = (XtArgVal)saveas_cbks;
  w = XtCreateManagedWidget("FM_Save_As",smeBSBObjectClass,file_menu_popup,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  w = XtCreateManagedWidget("Line_1",smeLineObjectClass,file_menu_popup,
			    NULL,0);

  menu_entry_args[0].value = (XtArgVal)"Quit";
  menu_entry_args[1].value = (XtArgVal)quit_cbks;
  w = XtCreateManagedWidget("FM_Quit",smeBSBObjectClass,file_menu_popup,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  /* Create Menu Button */
  w = XtCreateManagedWidget("File_Menu",menuButtonWidgetClass,form,
			    arg_list,XtNumber(arg_list));
  
  return;
}

static void new_data(Widget w,XtPointer client_data, XtPointer call_data)
{
  bzero(mem,num_kb*1024+34*sizeof(int));
  free(dlx_filename);
  dlx_filename = NULL;
  Write_Comm_Channel_Str("c");
  Reset_Output();
  XawMemListReset(data_list);
  XawMemListReset(code_list);
  Update_all();
  return;
}

static void open_data(Widget w,XtPointer client_data, XtPointer call_data)
{
  Window junk1,junk2;
  int root_x,root_y,junkX,junkY;
  unsigned int junkM;
  
  mode = OPEN_MODE;
  XtRealizeWidget(filename_popup);
  XQueryPointer(XtDisplay(filename_popup),XtWindow(filename_popup),
		&junk1,&junk2,&root_x,&root_y,&junkX,&junkY,&junkM);
  XtMoveWidget(filename_popup,root_x,root_y);
  XtPopup(filename_popup,XtGrabExclusive);
  return;
}

static void save_data(Widget w,XtPointer client_data, XtPointer call_data)
{
  mode = SAVE_MODE;

  if (dlx_filename)
/*    save(dlx_filename); */
    ;
  else
    saveas_data(w,client_data,call_data);

  return;
}

static void saveas_data(Widget w,XtPointer client_data, XtPointer call_data)
{
  Window junk1,junk2;
  int root_x,root_y,junkX,junkY;
  unsigned int junkM;
  
  mode = SAVE_MODE;
  XtRealizeWidget(filename_popup);
  XQueryPointer(XtDisplay(filename_popup),XtWindow(filename_popup),
		&junk1,&junk2,&root_x,&root_y,&junkX,&junkY,&junkM);
  XtMoveWidget(filename_popup,root_x,root_y);
  XtPopup(filename_popup,XtGrabExclusive);
  return;
}

static void quit_data(Widget w,XtPointer client_data, XtPointer call_data)
{
  kill(sim_pid,9);
  exit(0);
}

static void open_ok(Widget w,XtPointer client_data, XtPointer call_data)
{
  XtPopdown(XtParent(w));
  if (mode == OPEN_MODE)
    {
      bzero(mem,num_kb*1024+34*sizeof(int));
      dlx_filename = (char *)malloc(strlen(call_data)+1);
      strcpy(dlx_filename,call_data);
      load(dlx_filename);
      Write_Comm_Channel_Str("c");
      Reset_Output();
      XawMemListReset(data_list);
      XawMemListReset(code_list);
      Update_all();
    }
  else
    {
      dlx_filename = (char *)malloc(strlen(call_data)+1);
      strcpy(dlx_filename,call_data);
/*      save(dlx_filename); */
    }
}

static void open_cancel(Widget w,XtPointer client_data, XtPointer call_data)
{
  XtPopdown(XtParent(w));
}
